#!/usr/bin/python
# -*- coding: utf-8 -*-

__version__ = (0, 8, 0)
__author__ = "Rafał Furmański"
__contact__ = "r.furmanski@gmail.com"
__homepage__ = "http://github.com/r4fek/django-cassandra-engine"
__docformat__ = "markdown"



from threading import Lock

from cassandra.cluster import Session
from cassandra.cqlengine import connection
from cassandra.auth import PlainTextAuthProvider


class Cursor(object):
    def __init__(self, connection):
        self.connection = connection

    def execute(self, *args, **kwargs):
        return self.connection.execute(*args, **kwargs)

    def close(self):
        pass

    def fetchmany(self, _):
        return []


class FakeConnection(object):

    def commit(self):
        pass

    def rollback(self):
        pass

    def cursor(self):
        return Cursor(None)

    def close(self):
        pass


class CassandraConnection(object):

    def __init__(self, **options):

        self.hosts = options.get('HOST').split(',')
        self.keyspace = options.get('NAME')
        self.user = options.get('USER')
        self.password = options.get('PASSWORD')
        self.options = options.get('OPTIONS', {})
        self.connection_options = self.options.get('connection', {})
        self.session_options = self.options.get('session', {})

        if self.user and self.password and \
                'auth_provider' not in self.connection_options:
            self.connection_options['auth_provider'] = \
                PlainTextAuthProvider(username=self.user,
                                      password=self.password)

        self.lock = Lock()
        self.session = None
        self.cluster = None
        self.setup()

    def setup(self):
        with self.lock:
            self.session = connection.get_session()
            if not (self.session is None or self.session.is_shutdown):
                # already connected
                return

            for option, value in self.session_options.items():
                setattr(Session, option, value)
            connection.setup(self.hosts, self.keyspace,
                             **self.connection_options)
            self.session = connection.get_session()
            self.cluster = connection.get_cluster()

    def commit(self):
        pass

    def rollback(self):
        pass

    def cursor(self):
        return Cursor(self)

    def execute(self, qs, *args, **kwargs):
        self.session.set_keyspace(self.keyspace)
        return self.session.execute(qs, *args, **kwargs)

    def close(self):
        """ We would like to keep connection always open by default """

    def close_all(self):
        """
        Close all db connections
        """
        with self.lock:
            if self.cluster is not None:
                self.cluster.shutdown()

            if self.session is not None and not self.session.is_shutdown:
                self.session.shutdown()

            self.session = None
            self.cluster = None



import django

if django.VERSION[0:2] >= (1, 8):
    from django.db.backends.base.features import BaseDatabaseFeatures
else:
    from django.db.backends import BaseDatabaseFeatures


class CassandraDatabaseFeatures(BaseDatabaseFeatures):
    string_based_auto_field = True
    supports_long_model_names = False
    supports_microsecond_precision = True
    can_rollback_ddl = True
    uses_savepoints = False
    requires_rollback_on_dirty_transaction = False
    atomic_transactions = True

    # Django 1.4 compatibility
    def _supports_transactions(self):
        return False

    supports_transactions = _supports_transactions



from django.core.management.commands.flush import Command as FlushCommand

from django_cassandra_engine.utils import get_engine_from_db_alias


class Command(FlushCommand):

    def handle_noargs(self, **options):
        engine = get_engine_from_db_alias(options['database'])
        if engine == 'django_cassandra_engine':
            options.update({
                'load_initial_data': False,
                'interactive': False,
                'inhibit_post_migrate': True
            })

        return super(Command, self).handle_noargs(**options)

    def handle(self, **options):
        engine = get_engine_from_db_alias(options['database'])
        if engine == 'django_cassandra_engine':
            options.update({
                'load_initial_data': False,
                'interactive': False,
                'inhibit_post_migrate': True
            })

        return super(Command, self).handle(**options)

    @staticmethod
    def emit_post_syncdb(verbosity, interactive, database):
        engine = get_engine_from_db_alias(database)
        if engine != 'django_cassandra_engine':
            return FlushCommand.emit_post_syncdb(verbosity, interactive,
                                                 database)

    @staticmethod
    def emit_post_migrate(verbosity, interactive, database):
        engine = get_engine_from_db_alias(database)
        if engine != 'django_cassandra_engine':
            return FlushCommand.emit_post_migrate(verbosity, interactive,
                                                  database)



import django
if django.VERSION[0:2] >= (1, 8):
    from django.db.backends.base.operations import BaseDatabaseOperations
else:
    from django.db.backends import BaseDatabaseOperations


class CassandraDatabaseOperations(BaseDatabaseOperations):

    def pk_default_value(self):
        """
        Returns None, to be interpreted by back-ends as a request to
        generate a new key for an "inserted" object.
        """
        return None

    def quote_name(self, name):
        """
        Does not do any quoting, as it is not needed for most NoSQL
        databases.
        """
        return name

    def prep_for_like_query(self, value):
        """
        Does no conversion, parent string-cast is SQL specific.
        """
        return value

    def prep_for_iexact_query(self, value):
        """
        Does no conversion, parent string-cast is SQL specific.
        """
        return value

    def sql_flush(self, style, tables, sequences, allow_cascade=False):
        """
        Truncate all existing tables in current keyspace.

        :returns: an empty list
        """

        for table in tables:
            qs = "TRUNCATE {}".format(table)
            self.connection.connection.execute(qs)

        return []



try:
    from django.db.backends.base.validation import BaseDatabaseValidation
except ImportError:
    from django.db.backends import BaseDatabaseValidation


class CassandraDatabaseValidation(BaseDatabaseValidation):
    pass



import django
if django.VERSION[0:2] >= (1, 8):
    from django.db.backends.base.schema import BaseDatabaseSchemaEditor
else:
    from django.db.backends.schema import BaseDatabaseSchemaEditor


class CassandraDatabaseSchemaEditor(BaseDatabaseSchemaEditor):

    def prepare_default(self, value):
        return value

    def quote_value(self, value):
        return value

    def create_model(self, model):
        # Hack to disable checking migrations for cassandra connection
        try:
            from django.db.migrations.exceptions import (
                MigrationSchemaMissing as Exc)
        except ImportError:
            from django.core.exceptions import ImproperlyConfigured as Exc
        raise Exc('No schema for cassandra database')

    def delete_model(self, model):
        pass



try:
    from django.core.management.commands.syncdb import Command as SyncCommand
except ImportError:
    from django_cassandra_engine.management.commands.sync_cassandra import Command as SyncCommand

from django_cassandra_engine.management.commands import sync_cassandra
from django_cassandra_engine.utils import get_engine_from_db_alias


class Command(SyncCommand):

    def handle_noargs(self, **options):
        engine = get_engine_from_db_alias(options['database'])

        # Call regular syncdb if engine is different from ours
        if engine != 'django_cassandra_engine':
            return super(Command, self).handle_noargs(**options)
        else:
            return sync_cassandra.Command().execute(**options)

    def handle(self, **options):
        engine = get_engine_from_db_alias(options['database'])

        # Call regular syncdb if engine is different from ours
        if engine != 'django_cassandra_engine':
            return super(Command, self).handle(**options)
        else:
            return sync_cassandra.Command().execute(**options)



from django.core.management import call_command
from django.core.management.commands.migrate import Command as MigrateCommand

from django_cassandra_engine.utils import get_engine_from_db_alias


class Command(MigrateCommand):

    def handle(self, *args, **options):
        engine = get_engine_from_db_alias(options['database'])

        # Call regular migrate if engine is different from ours
        if engine != 'django_cassandra_engine':
            return super(Command, self).handle(*args, **options)
        else:
            self.stdout.write("Migrations are not supported in this engine. "
                              "Calling syncdb instead..")
            call_command('sync_cassandra', **options)
